@@ -7,8 +7,6 @@ from pyceg.Multiprocessor.Operations import MSIOps as Ops
 ###################
 # MSI Protocol
 ###################
-
-
 class MSI(Protocol):
     def __init__(self, n_processors=2, memory_content=None):
         super(MSI, self).__init__(n_processors, memory_content)
@@ -40,30 +38,11 @@ class MSI(Protocol):
             elif event == Ops.PrRd:
                 state = State.S
                 transaction = Ops.BusRd
-        # Set State
+        # Set New State
         self.set_processor_state(pid, state)
 
-        # Processor Write to Cache using an Operation
-        if event == Ops.PrWr:
-            self.perform_processor_operation(pid, function)
-
-        # Self Invalidate
-        if event == Ops.PrInv:
-            self.set_processor_state(pid, "I")
-            self.set_processor_cache_content(pid, None)
-
-        # Invalidate copy
-        if event in (Ops.BusUpgr, Ops.BusRdX):
-            self.set_processor_cache_content(pid, None)
-
-        # If the new state is modified, store the modified value
-        if state == State.M:
-            self.modified_value = self.get_processor_cache_content(pid)
-
-        # Processor Read/Write from Memory
-        if event in (Ops.PrRd, Ops.PrWr):
-            cache_content = self.modified_value or self.memory_content
-            self.set_processor_cache_content(pid, cache_content)
+        # Perform Post Event Logic, to set Processor's Cache and Memory Content
+        super().on_post_event(pid=pid, event=event, function=function)
 
         return transaction
 
@@ -79,12 +58,11 @@ def example1():
         (2, Ops.PrRd),
         (2, Ops.PrWr, lambda x: x + 2),
         (1, Ops.PrWr, lambda x: x * 2),
-        # (1, Ops.PrInv),
         (2, Ops.PrRd),
     ]
-    msi = MSI(n_processors=2, memory_content=3)
+    msi = MSI(n_processors=2, memory_content=10)
     msi.perform_instructions(instructions)
-    print(msi.statistics())
+
 
 ###################
 # Main
