@@ -1,7 +1,7 @@
-from .MSI import MSI
-from .MESI import MESI
-from .Operations import MSIOps as Ops
-from .CacheCoherence import protocols
+from MSI import MSI
+from MESI import MESI
+from Operations import MSIOps as Ops
+from CacheCoherence import protocols
 
 
 ###################
@@ -76,13 +76,10 @@ def example4():
     msi = MSI(n_processors=2, memory_content=10)
     msi.perform_instructions(instructions)
 
-    # print()
     # msi = MSI(n_processors=2, memory_content=10)
     # msi.perform_instructions(instructions)
 
 
-# print(example4())
-
 ###################
 # Example 5
 # DGD 5 Fall 2020 and Quiz 2 2019
@@ -118,7 +115,7 @@ def example6():
 
 
 ###################
-# http://people.ee.duke.edu/~sorin/prior-courses/ece152-spring2009/lectures/8.3-multicore.pdf
+# Multiple Protocols Load Testing
 ###################
 def example7():
     instructions = [
@@ -127,13 +124,13 @@ def example7():
         (3, Ops.PrWr, lambda x: x + 2),
         (1, Ops.PrRd),
         (2, Ops.PrRd),
+        (2, Ops.PrWr, lambda x: x + 2),
     ]
     for name, protocol in protocols.items():
         p = protocol(n_processors=3, memory_content=3)
         p.perform_instructions(instructions, display=False)
         print(p.statistics())
 
-
 ###################
 # Main
 ###################
