@@ -1,23 +1,16 @@
 from pyceg.Multiprocessor.Processor import Processor
 from pyceg.Multiprocessor.Operations import MESIOps as Ops
-from pyceg.Multiprocessor.States import MESIState as State
+from pyceg.Multiprocessor.States import MOESIState as State, VIState
+from pyceg.Multiprocessor.Statistics import Statistics
 from tabulate import tabulate
 
 
 class Protocol:
     def __init__(self, n_processors=2, memory_content=None):
-        self.n_processors = n_processors
         self.processors = [Processor(pid=i + 1) for i in range(n_processors)]
         self.memory_content = memory_content
-        # Statistical Parameters
-        self.n_reads = 0
-        self.n_writes = 0
-        self.n_state_changes = 0
-        self.n_invalidations = 0
-        self.n_bus_latency = 0
-        self.n_cache_misses = 0
-        self.n_cache_to_cache_transfers = 0
-
+        # Storing Statistical Parameters
+        self.stats = Statistics()
         # Additional Private Variables
         self.modified_value = None
         self.history = []
@@ -30,30 +23,50 @@ class Protocol:
 
     def on_event(self, pid, event, function):
         if event == Ops.PrRd:
-            self.n_reads += 1
+            self.stats.n_reads += 1
         elif event == Ops.PrWr:
-            self.n_writes += 1
+            self.stats.n_writes += 1
         state = self.get_processor_state(pid)
         if state == State.I:
             states = {p.state for p in self.other_processors_not(pid=pid)}
             if len(states) == 1 and State.I in states:
-                self.n_cache_misses += 1
+                self.stats.n_cache_misses += 1
             if State.S in states:
-                self.n_cache_misses += 1
-            if states.intersection({State.M, State.E}):
-                self.n_cache_to_cache_transfers += 1
+                self.stats.n_cache_misses += 1
+            if states.intersection({State.M, State.E, State.O}):
+                self.stats.n_cache_to_cache_transfers += 1
+        return ''
 
-    def get_headers(self):
-        headers = ["Step"]
-        [headers.extend(["P{} State".format(i + 1), "P{} Cache".format(i + 1)]) for i, p in enumerate(self.processors)]
-        headers.extend(["Memory Content", "Bus Transaction", "Modified"])
-        return headers
+    def on_post_event(self, pid, event, function):
+        """
+        Perform Post Event Logic, to set Processor's Cache and Memory Content
+        """
+        state = self.get_processor_state(pid)
+        # Processor Write to Cache using an Operation
+        if event == Ops.PrWr:
+            self.perform_processor_operation(pid, function)
 
-    def save_history(self, step_name, transaction):
-        fields = [step_name]
-        [fields.extend([p.state, p.cache_content or '-']) for p in self.processors]
-        fields.extend([self.memory_content or '-', transaction or '-', self.modified_value or '-'])
-        self.history.append(fields)
+        # Self Invalidate
+        if event == Ops.PrInv:
+            self.set_processor_state(pid, "I")
+            self.set_processor_cache_content(pid, None)
+
+        # Invalidate copy
+        if event in (Ops.BusUpgr, Ops.BusRdX):
+            self.set_processor_cache_content(pid, None)
+
+        # If the new state is modified, store the modified value
+        if state in (State.M, VIState.V):
+            self.modified_value = self.get_processor_cache_content(pid)
+
+        # Processor Read/Write from Memory
+        if event in (Ops.PrRd, Ops.PrWr):
+            cache_content = self.modified_value or self.memory_content
+            self.set_processor_cache_content(pid, cache_content)
+
+    def all_processors_are_invalid(self):
+        """ MESI Extensions"""
+        return all(p.state == State.I for p in self.processors)
 
     def processor(self, pid):
         return self.processors[pid - 1]
@@ -67,9 +80,8 @@ class Protocol:
 
     def set_processor_state(self, pid, state):
         if self.processor(pid).state != state:
-            self.n_state_changes += 1
-            if state == "I":
-                self.n_invalidations += 1
+            self.stats.n_state_changes += 1
+            if state == "I": self.stats.n_invalidations += 1
         self.processor(pid).state = state
 
     def get_processor_cache_content(self, pid):
@@ -89,11 +101,13 @@ class Protocol:
         Places the value of value of the cache line on the bus and updates the memory
         """
         self.memory_content = value
+        self.stats.n_flushes += 1
 
     def _perform_instruction(self, pid, event, function):
         # Perform the current Processor Event
         main_transaction = self.on_event(pid, event, function)
-        if main_transaction: self.n_bus_latency += 1
+        if main_transaction: self.stats.n_bus_latency += 1
+        all_transactions = [main_transaction] if main_transaction else []
         # Update the other processors
         for processor in self.processors:
             if processor.pid != pid:
@@ -101,12 +115,12 @@ class Protocol:
                 transaction = self.on_event(processor.pid, main_transaction, function)
                 if transaction == Ops.Flush:
                     self.flush(cache_content)
-                    main_transaction += "/" + Ops.Flush
-                if transaction: self.n_bus_latency += 1
+                    all_transactions.append(Ops.Flush)
+                if transaction: self.stats.n_bus_latency += 1
 
         # Save the current state into history for printing
         step_name = self.format_instruction_name(pid, event)
-        self.save_history(step_name, main_transaction)
+        self.save_history(step_name, '/'.join(all_transactions))
 
     def perform_instructions(self, instructions, display=True):
         for i in instructions:
@@ -114,21 +128,33 @@ class Protocol:
             self._perform_instruction(pid, operation, function)
         if display: print(self, '\n')
 
+    ###########################
+    # Display
+    ###########################
+    def get_headers(self):
+        headers = ["Step"]
+        [headers.extend(["P{} State".format(i + 1), "P{} Cache".format(i + 1)]) for i, p in enumerate(self.processors)]
+        headers.extend(["Memory Content", "Bus Transaction", "Modified"])
+        return headers
+
+    def save_history(self, step_name, transaction):
+        fields = [step_name]
+        [fields.extend([p.state, p.cache_content or '-']) for p in self.processors]
+        fields.extend([self.memory_content or '-', transaction or '-', self.modified_value or '-'])
+        self.history.append(fields)
+
+    ###########################
+    # API Interface
+    ###########################
+    def api(self):
+        """Protocol API Interface"""
+        return {"title": self.get_headers(), "results": [item[:-1] for item in self.history], "table": str(self), "stats": self.statistics()}
+
     ###########################
     # Statistical Methods
     ###########################
     def statistics(self):
-        stats = {
-            "Instructions": self.step_count(),
-            "Reads": self.n_reads,
-            "Writes": self.n_writes,
-            "Bus Transactions": self.n_bus_latency,
-            "States": self.n_state_changes,
-            "Invalidations": self.n_invalidations,
-            "Cache-to-Cache Transfers": self.n_cache_to_cache_transfers,
-            "Cache Misses": self.n_cache_misses,
-        }
-        return stats
+        return {"Instructions:": self.step_count(), **self.stats.info()}
 
     ###########################
     # Additional Helper Methods
@@ -137,34 +163,16 @@ class Protocol:
         return len(self.history)
 
     def format_instruction_name(self, processor_id, event):
-        return "{}. P{} {}".format(len(self.history) + 1, processor_id, event)
+        return "{}. P{} {}".format(self.step_count() + 1, processor_id, event)
 
     @staticmethod
     def format_instruction_set(func_name):
         if not func_name: return ""
-        import inspect
-        code = inspect.getsourcelines(func_name)[0][0].strip()
-        code = code.replace("(", "").replace(")", "").split(",")[2].strip()
-        code = code.split("lambda ")[1]
-        return code
+        code = __import__("inspect").getsourcelines(func_name)[0][0].strip()
+        return code.replace("(", "").replace(")", "").split(",")[2].strip().split("lambda ")[1]
 
     def print(self, _filter=None):
-        _filter = self.filter_dict().get(_filter, lambda x: x)
-        headers = _filter(self.get_headers())
-        history = [_filter(x) for x in self.history]
+        filter_dict = {"cache": lambda v: v[:1] + v[1:][:-3:2], "cache-transaction": lambda v: filter_dict["cache"](v) + [v[-2]]}
+        _filter = filter_dict.get(_filter, lambda x: x)
+        headers, history = _filter(self.get_headers()), [_filter(x) for x in self.history]
         print(tabulate(history, headers=headers, stralign="center", numalign="center"))
-
-    def api(self):
-        """Protocol API Interface"""
-        return {"title": self.get_headers(), "results": [item[:-1] for item in self.history], "table": str(self), "stats": self.statistics()}
-
-    def filter_dict(self):
-        return {"cache": self._field_cache_filter, "transaction": self._field_transaction_filter}
-
-    @staticmethod
-    def _field_cache_filter(array):
-        return array[:1] + array[1:][:-3:2]
-
-    @staticmethod
-    def _field_transaction_filter(array):
-        return array[:1] + array[1:][:-3:2] + [array[-2]]
